USE [AdventureWorks]
GO

/****** Object:  StoredProcedure [Production].[Add_BOM_2]    Script Date: 03/16/2012 11:54:51 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO


/*
2012-03-14	BOM sample

*/
CREATE proc [Production].[Add_BOM_2] (@Parent as int, @Child as int, @BOMQty decimal (8,2), @UoM nchar(3)) as
begin

declare @StartDate datetime; set @StartDate = GETDATE() -- BOM Start Date
declare @ModifiedDate datetime ; set @ModifiedDate = GETDATE() -- BOM Modified Date
declare @BOMLevel smallint; -- BOM level 
declare @SellEndDate datetime -- Sales end date
declare @DiscontinuedDate datetime -- Components discontined date
declare @Name as nvarchar(50) -- Components description 

-- Look up components level in the BOM to be used when inserting new BOM line
select top 1 @BOMLevel = BOMLevel from Production.BillOfMaterials 
where EndDate is null and ProductAssemblyID = @Parent

-- Look up Sales End Date, discontinued date and description from Product table
select @SellEndDate = SellEndDate, @DiscontinuedDate = DiscontinuedDate, @Name = Name 
from Production.Product where ProductID = @Child

-- Abort and raise error if parent and child combination already exist in BOM
if exists(select 1 from Production.BillOfMaterials 
where ProductAssemblyID = @Parent and ComponentID = @Child) 
begin raiserror('Item already exist in BOM',11,1) with seterror; return 1 end

-- Insert new BOM line
insert Production.BillOfMaterials 
(ProductAssemblyID, ComponentID, StartDate, UnitMeasureCode, BOMLevel, PerAssemblyQty, ModifiedDate)
values (@Parent, @Child, @StartDate, @UoM, @BOMLevel, @BOMQty, @ModifiedDate)

-- Validate whether to send a notification or not
if @SellEndDate is not null or @DiscontinuedDate is not null 

begin

	-- Begin Email Notification
	declare @gKey uniqueidentifier; set @gKey = newid(); -- Boomerang Key
	declare @jKey uniqueidentifier; set @jKey = newid(); -- Boomerang Key
	declare @subj varchar(100); set @subj = 'BOM Notification'; -- Email Subject
	declare @body varchar(1500); -- Email Body

	-- Set email body. Add lines if sales end date and discontinued date exist
	set @body = 'This is to notify you that the component ' + convert(varchar(10), @Child)+ ' | ' + @Name + ' added to BOM '
				+ convert(varchar(10), @Parent) + ' on ' + convert(varchar(20), @ModifiedDate, 101) + ' needs to be replaced before: ' + CONVERT(varchar(20), @DiscontinuedDate, 101) + char(13) + char(10);
				if(@SellEndDate is not null) set @body = @body + char(13) + char(10) + '* component has a Sell End Date of ' + convert(varchar (20), @SellEndDate, 101);
				if(@DiscontinuedDate is not null) set @body = @body + char(13) + char(10) + '* component will be Discontinued on ' + CONVERT(varchar(20), @DiscontinuedDate, 101);
	print @body
	-- Create new event by insert row in event_master
	insert Boomerang..EVENT_MASTER (gKey, Str1, Created_By, Source)
	values (@gKey, 'BOM Notification', 'Production.Add_BOM', 4);

	-- Create email our event 
	insert Boomerang..OUT_EMAIL (gKey, jKey, [From], Subject, Body, IncludeKey)
	values(@gKey, @jKey, '"Production Notification Services" <info@fuel9.com>', @subj, @body, 0);

	-- add recipients to email 
	exec Boomerang..sp_Add_Email_List @jKey, 'info@fuel9.com'

	-- Set delayed delivery date and time for notification
	update Boomerang..EVENT_STATUS set Run_When = dateadd(day, -1, @DiscontinuedDate) where gKey = @gKey and jKey = @jKey -- send message 1 day before being discontinued

	-- release event
	update Boomerang..EVENT_MASTER set Status=0 where gKey=@gKey;

end

end


/*
select * from Production.BillOfMaterials where ProductAssemblyID in (732)
select * from Production.Product where ProductID=732

exec Production.Add_BOM_2 732, 845, 10, 'EA'
delete from Production.BillOfMaterials where ProductAssemblyID = 732 and ComponentID=845
update Production.Product  set DiscontinuedDate = getdate()+150 where ProductID = 845

*/

GO

